@@ -41,7 +41,7 @@ void spawn(void *data);
 char * command (int interactive, char *data);
 
 char * cmd_newwm(int interactive, void *which);
-char * cmd_generate (int interactive, void *data);
+char * cmd_meta (int interactive, void *data);
 char * cmd_abort (int interactive, void *data);
 char * cmd_exec (int interactive, void *data);
 char * cmd_colon (int interactive, void *data);
@@ -56,12 +56,12 @@ char * cmd_prev (int interactive, void *data);
 char * cmd_prev_frame (int interactive, void *data);
 char * cmd_windows (int interactive, void *data);
 char * cmd_other (int interactive, void *data);
-char * cmd_clock (int interactive, void *data);
+char * cmd_time (int interactive, void *data);
 char * cmd_version (int interactive, void *data);
 char * cmd_unimplemented (int interactive, void *data);
 char * cmd_bind (int interactive, void* data);
 char * cmd_source (int interactive, void* data);
-char * cmd_maximize (int interactive, void *data);
+char * cmd_redisplay (int interactive, void *data);
 char * cmd_escape (int interactive, void *data);
 char * cmd_echo (int interactive, void *data);
 char * cmd_h_split (int interactive, void *data);
@@ -75,8 +75,39 @@ char * cmd_quit(int interactive, void *data);
 char * cmd_number (int interactive, void *data);
 char * cmd_rudeness (int interactive, void *data);
 char * cmd_unbind (int interactive, void *data);
-
-/* void cmd_xterm (void *data); */
+char * cmd_gravity (int interactive, void *data);
+char * cmd_defwingravity (int interactive, void *data);
+char * cmd_deftransgravity (int interactive, void *data);
+char * cmd_defmaxsizegravity (int interactive, void *data);
+char * cmd_msgwait (int interactive, void *data);
+char * cmd_defbargravity (int interactive, void *data);
+char * cmd_deffont (int interactive, void *data);
+char * cmd_defpadding (int interactive, void *data);
+char * cmd_defborder (int interactive, void *data);
+char * cmd_definputwidth (int interactive, void *data);
+char * cmd_defwaitcursor (int interactive, void *data);
+char * cmd_defwinfmt (int interactive, void *data);
+char * cmd_defwinname (int interactive, void *data);
+char * cmd_deffgcolor (int interactive, void *data);
+char * cmd_defbgcolor (int interactive, void *data);
+char * cmd_setenv (int interactive, void *data);
+char * cmd_getenv (int interactive, void *data);
+char * cmd_chdir (int interactive, void *data);
+char * cmd_unsetenv (int interactive, void *data);
+char * cmd_info (int interactive, void *data);
+char * cmd_lastmsg (int interactive, void *data);
+char * cmd_focusup (int interactive, void *data);
+char * cmd_focusdown (int interactive, void *data);
+char * cmd_focusleft (int interactive, void *data);
+char * cmd_focusright (int interactive, void *data);
+char * cmd_restart (int interactive, void *data);
+char * cmd_startup_message (int interactive, void *data);
+char * cmd_focuslast (int interactive, void *data);
+char * cmd_link (int interactive, void *data);
+char * cmd_defbarpadding (int interactive, void *data);
+char * cmd_license (int interactive, void *data);
+char * cmd_alias (int interactive, void *data);
+char *cmd_defbarborder (int interactive, void *data);
 
 void initialize_default_keybindings (void);
 rp_action* find_keybinding (KeySym keysym, int state);
