@@ -40,19 +40,26 @@ struct rp_window_frame
   int x, y, width, height;
   rp_window *win;
 
+  /* For determining the last frame. */
+  int last_access;
+
   rp_window_frame *prev, *next;
 };
 
-
 struct rp_window
 {
   screen_info *scr;
   Window w;
   int number;
-  char *name;
   int state;
-  int last_access;		
+  int last_access;
   int named;
+  
+  /* Window name hints. */
+  char *user_name;
+  char *wm_name;
+  char *res_name;
+  char *res_class;
 
   /* Dimensions */
   int x, y, width, height, border;
@@ -70,6 +77,10 @@ struct rp_window
   /* Saved mouse position */
   int mouse_x, mouse_y;
 
+  /* The alignment of the window. Decides to what side or corner the
+     window sticks to. */
+  int gravity;
+
   /* A window can be visible inside a frame but not the frame's
      current window. This keeps track of what frame the window was
      mapped into. */
@@ -81,13 +92,23 @@ struct rp_window
 struct screen_info
 {
   GC normal_gc;
-  XFontStruct *font;		/* The font we want to use. */
   XWindowAttributes root_attr;
   Window root, bar_window, key_window, input_window, frame_window, help_window;
   int bar_is_raised;
   int screen_num;		/* Our screen number as dictated my X */
   Colormap def_cmap;
   Cursor rat;
+  unsigned long fg_color, bg_color; /* The pixel color. */
+
+  char *display_string;
+
+  /* A list of frames that may or may not contain windows. There should
+     always be one in the list. */
+  rp_window_frame *rp_window_frame_sentinel;
+
+  /* Pointer to the currently focused frame. One for each screen so
+     when you switch screens the focus doesn't get frobbed. */
+  rp_window_frame *rp_current_frame;
 };
 
 struct rp_action
@@ -104,6 +125,44 @@ struct rp_key
   unsigned int state;
 };
 
+struct rp_defaults
+{
+  /* Default positions for new normal windows, transient windows, and
+     normal windows with maxsize hints. */
+  int win_gravity;
+  int trans_gravity;
+  int maxsize_gravity;
+
+  int input_window_size;
+  int window_border_width;
+
+  int bar_x_padding;
+  int bar_y_padding;
+  int bar_location;
+  int bar_timeout;
+  int bar_border_width;
+  
+  int frame_indicator_timeout;
+
+  int padding_left;
+  int padding_right;
+  int padding_top;
+  int padding_bottom;
+
+  XFontStruct *font;
+
+  int wait_for_key_cursor;
+
+  char *window_fmt;
+
+  /* Which name to use: wm_name, res_name, res_class. */
+  int win_name;
+
+  int startup_message;
+};
+
+extern struct rp_defaults defaults;
+
 /* The prefix key also known as the command character under screen. */
 extern struct rp_key prefix_key;
 
@@ -125,26 +184,17 @@ extern rp_window *rp_mapped_window_sentinel;
    assigned to them and are not visible/active. */
 extern rp_window *rp_unmapped_window_sentinel;
 
-/* A list of frames that may or may not contain windows. There should
-   always be one in the list. */
-extern rp_window_frame *rp_window_frame_sentinel;
-
-/* Pointer to the currently focused frame. */
-extern rp_window_frame *rp_current_frame;
-
+extern int rp_current_screen;
 extern screen_info *screens;
 extern int num_screens;
 
 extern XEvent rp_current_event;
 
 extern Display *dpy;
-extern Atom rp_restart;
-extern Atom rp_kill;
 extern Atom rp_command;
 extern Atom rp_command_request;
 extern Atom rp_command_result;
 
-
 extern Atom wm_state;
 extern Atom wm_change_state;
 extern Atom wm_protocols;
@@ -165,6 +215,15 @@ extern int ignore_badwindow;
 /* Arguments passed to ratpoison. */
 extern char **myargv;
 
+/* These defines should be used to specify the modifier mask for keys
+   and they are translated into the X11 modifier mask when the time
+   comes to compare modifier masks. */
+#define RP_CONTROL_MASK 1
+#define RP_META_MASK 	2
+#define RP_ALT_MASK 	4
+#define RP_SUPER_MASK 	8
+#define RP_HYPER_MASK 	16
+
 struct modifier_info
 {
 /*   unsigned int mode_switch_mask; */
@@ -194,4 +253,7 @@ extern int rp_honour_normal_raise;
 extern int rp_honour_transient_map;
 extern int rp_honour_normal_map;
 
+/* Keep track of X11 error messages. */
+extern char *rp_error_msg;
+
 #endif /* _RATPOISON_DATA_H */
