@@ -27,6 +27,7 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 #include <signal.h>
 #include <errno.h>
 #include <unistd.h>
@@ -112,16 +113,16 @@ unmap_notify (XEvent *ev)
   switch (win->state)
     {
     case IconicState:
-      PRINT_DEBUG ("Withdrawing iconized window '%s'\n", win->name);
+      PRINT_DEBUG ("Withdrawing iconized window '%s'\n", window_name (win));
       if (ev->xunmap.send_event) withdraw_window (win);
       break;
     case NormalState:
-      PRINT_DEBUG ("Withdrawing normal window '%s'\n", win->name);
+      PRINT_DEBUG ("Withdrawing normal window '%s'\n", window_name (win));
       /* If the window was inside a frame, fill the frame with another
 	 window. */
       frame = find_windows_frame (win);
       if (frame) cleanup_frame (frame);
-      if (frame == rp_current_frame) set_active_frame (frame);
+      if (frame == win->scr->rp_current_frame) set_active_frame (frame);
 
       withdraw_window (win);
       break;
@@ -182,10 +183,10 @@ map_request (XEvent *ev)
 	    {
 	      if (win->transient)
 		marked_message_printf (0, 0, MESSAGE_RAISE_TRANSIENT, 
-				       win->number, win->name);
+				       win->number, window_name (win));
 	      else
 		marked_message_printf (0, 0, MESSAGE_RAISE_WINDOW,
-				       win->number, win->name);
+				       win->number, window_name (win));
 	    }
 	}
       break;
@@ -208,26 +209,13 @@ destroy_window (XDestroyWindowEvent *ev)
      case... */
   frame = find_windows_frame (win);
   if (frame) cleanup_frame (frame);
-  if (frame == rp_current_frame) set_active_frame (frame);
+  if (frame == win->scr->rp_current_frame) set_active_frame (frame);
 
   unmanage (win);
 
   ignore_badwindow--;
 }
 
-static void
-configure_notify (XConfigureEvent *e)
-{
-  rp_window *win;
-
-  win = find_window (e->window);
-  if (win)
-    {
-      PRINT_DEBUG ("'%s' window notify: %d %d %d %d %d\n", win->name,
-		   e->x, e->y, e->width, e->height, e->border_width);
-    }
-}
-
 static void
 configure_request (XConfigureRequestEvent *e)
 {
@@ -238,68 +226,53 @@ configure_request (XConfigureRequestEvent *e)
 
   if (win)
     {
-      /* Updated our window struct */
-      if (e->value_mask & CWX)
-	{
-	  changes.x = e->x + win->border;
-	  win->x = e->x + win->border;
-	  PRINT_DEBUG("request CWX %d\n", e->x);
-	}
-      if (e->value_mask & CWY)
-	{
-	  changes.y = e->y + win->border;
-	  win->y = e->y + win->border;
-	  PRINT_DEBUG("request CWY %d\n", e->y);
-	}
-
       if (e->value_mask & CWStackMode)
 	{
 	  if (e->detail == Above)
 	    {
 	      /* Depending on the rudeness level, actually map the
 		 window. */
-	      if (win->state == IconicState)
+	      if ((rp_honour_transient_raise && win->transient)
+		  || (rp_honour_normal_raise && !win->transient))
 		{
-		  if ((rp_honour_transient_raise && win->transient)
-		      || (rp_honour_normal_raise && !win->transient))
-		    {
-		      set_active_window (win);
-		    }
-		  else 
-		    {
-		      if (win->transient)
-			marked_message_printf (0, 0, MESSAGE_RAISE_TRANSIENT,
-					       win->number, win->name);
-		      else
-			marked_message_printf (0, 0, MESSAGE_RAISE_WINDOW,
-					       win->number, win->name);
-		    }
+		  if (win->state == IconicState)
+		    set_active_window (win);
+		  else if (find_windows_frame (win))
+		    goto_window (win);
 		}
 	      else
 		{
-		  if (find_windows_frame (win))
-		    goto_window (win);
+		  if (win->transient)
+		    marked_message_printf (0, 0, MESSAGE_RAISE_TRANSIENT,
+					   win->number, window_name (win));
+		  else
+		    marked_message_printf (0, 0, MESSAGE_RAISE_WINDOW,
+					   win->number, window_name (win));
 		}
+
 	    }
 
 	  PRINT_DEBUG("request CWStackMode %d\n", e->detail);
 	}
 
-      PRINT_DEBUG ("'%s' window size: %d %d %d %d %d\n", win->name,
+      PRINT_DEBUG ("'%s' window size: %d %d %d %d %d\n", window_name (win),
 		   win->x, win->y, win->width, win->height, win->border);
 
+      /* Collect the changes to be granted. */
       if (e->value_mask & CWBorderWidth)
 	{
 	  changes.border_width = e->border_width;
 	  win->border = e->border_width;
 	  PRINT_DEBUG("request CWBorderWidth %d\n", e->border_width);
 	}
+
       if (e->value_mask & CWWidth)
 	{
 	  changes.width = e->width;
 	  win->width = e->width;
 	  PRINT_DEBUG("request CWWidth %d\n", e->width);
 	}
+
       if (e->value_mask & CWHeight)
 	{
 	  changes.height = e->height;
@@ -307,27 +280,43 @@ configure_request (XConfigureRequestEvent *e)
 	  PRINT_DEBUG("request CWHeight %d\n", e->height);
 	}
 
+      if (e->value_mask & CWX)
+	{
+	  changes.x = e->x;
+	  win->x = e->x;
+	  PRINT_DEBUG("request CWX %d\n", e->x);
+	}
+
+      if (e->value_mask & CWY)
+	{
+	  changes.y = e->y;
+	  win->y = e->y;
+	  PRINT_DEBUG("request CWY %d\n", e->y);
+	}
+
       if (e->value_mask & (CWX|CWY|CWBorderWidth|CWWidth|CWHeight))
 	{
-	  if (win->state != NormalState)
-	    {
-	      /* The window isn't visible so grant it whatever it likes */
-	      XConfigureWindow (dpy, win->w, e->value_mask & (CWX|CWY|CWBorderWidth|CWWidth|CWHeight), 
-				&changes);
-	      send_configure (win);
-	    }
-	  else
-	    {
-	      /* Draw the hard line. Get back in line, you misbehaving
-		 window! */
-	      maximize (win);
-	      send_configure (win);
-	    }
+	  /* Grant the request, then immediately maximize it. */
+	  XConfigureWindow (dpy, win->w, 
+			    e->value_mask & (CWX|CWY|CWBorderWidth|CWWidth|CWHeight), 
+			    &changes);
+	  XSync(dpy, False);
+	  if (win->state == NormalState)
+	    maximize (win);
 	}
     }
   else
     {
-      PRINT_DEBUG ("FIXME: Don't handle this\n");
+      /* Its an unmanaged window, so give it what it wants. But don't
+	 change the stack mode.*/
+      if (e->value_mask & CWX) changes.x = e->x;
+      if (e->value_mask & CWY) changes.x = e->x;
+      if (e->value_mask & CWWidth) changes.x = e->x;
+      if (e->value_mask & CWHeight) changes.x = e->x;
+      if (e->value_mask & CWBorderWidth) changes.x = e->x;
+      XConfigureWindow (dpy, e->window, 
+			e->value_mask & (CWX|CWY|CWBorderWidth|CWWidth|CWHeight), 
+			&changes);
     }
 }
 
@@ -336,29 +325,29 @@ client_msg (XClientMessageEvent *ev)
 {
   PRINT_DEBUG ("Received client message.\n");
 
-  if (ev->message_type == rp_restart)
-    {
-      PRINT_DEBUG ("Restarting\n");
-      clean_up (); 
-      execvp(myargv[0], myargv);
-    }
-  else if (ev->message_type == rp_kill)
-    {
-      PRINT_DEBUG ("Exiting\n");
-      clean_up ();
-      exit (EXIT_SUCCESS);
-    }
-  else if (ev->message_type == wm_change_state)
+  if (ev->message_type == wm_change_state)
     {
       rp_window *win;
 
+      PRINT_DEBUG ("WM_CHANGE_STATE\n")
+
       win = find_window (ev->window);
       if (win == NULL) return;
       if (ev->format == 32 && ev->data.l[0] == IconicState)
 	{
+	  /* FIXME: This means clients can hide themselves without the
+	     user's intervention. This is bad, but Emacs is the only
+	     program I know of that iconifies itself and this is
+	     generally from the user pressing C-z.  */
+	  PRINT_DEBUG ("Iconify Request.\n");
 	  if (win->state == NormalState)
 	    {
-	      /* TODO: Handle iconify events */
+	      rp_window *w = find_window_other();
+
+	      if (w)
+		set_active_window (w);
+	      else
+		blank_frame (win->scr->rp_current_frame);
 	    }
 	}
       else
@@ -368,7 +357,6 @@ client_msg (XClientMessageEvent *ev)
     }
 }
 
-#ifdef USE_WAITFORKEY_CURSOR
 static void
 grab_rat ()
 {
@@ -376,15 +364,12 @@ grab_rat ()
 		GrabModeAsync, GrabModeAsync, 
 		None, current_screen()->rat, CurrentTime);
 }
-#endif
 
-#ifdef USE_WAITFORKEY_CURSOR
 static void
 ungrab_rat ()
 {
   XUngrabPointer (dpy, CurrentTime);
 }
-#endif
 
 static void
 handle_key (screen_info *s)
@@ -395,11 +380,12 @@ handle_key (screen_info *s)
   Window fwin;			/* Window currently in focus */
   KeySym keysym;		/* Key pressed */
   unsigned int mod;		/* Modifiers */
+  int rat_grabbed = 0;
 
   PRINT_DEBUG ("handling key...\n");
 
   /* All functions hide the program bar and the frame indicator. */
-  if (BAR_TIMEOUT > 0) hide_bar (s);
+  if (defaults.bar_timeout > 0) hide_bar (s);
   hide_frame_indicator();
 
   /* Disable any alarm that was going to go off. */
@@ -411,16 +397,18 @@ handle_key (screen_info *s)
 
   /* Change the mouse icon to indicate to the user we are waiting for
      more keystrokes */
-#ifdef USE_WAITFORKEY_CURSOR
-  grab_rat();
-#endif
+  if (defaults.wait_for_key_cursor)
+    {
+      grab_rat();
+      rat_grabbed = 1;
+    }
 
   read_key (&keysym, &mod, NULL, 0);
+  XSetInputFocus (dpy, fwin, revert, CurrentTime);
 
-  if ((key_action = find_keybinding (keysym, mod)))
+  if ((key_action = find_keybinding (keysym, x11_mask_to_rp_mask (mod))))
     {
       char *result;
-      XSetInputFocus (dpy, fwin, revert, CurrentTime);
       result = command (1, key_action->data);
       
       /* Gobble the result. */
@@ -430,16 +418,13 @@ handle_key (screen_info *s)
   else
     {
       /* No key match, notify user. */
-      XSetInputFocus (dpy, fwin, revert, CurrentTime);
-
-      keysym_name = keysym_to_string (keysym, mod);
+      keysym_name = keysym_to_string (keysym, x11_mask_to_rp_mask (mod));
       marked_message_printf (0, 0, " %s unbound key ", keysym_name);
       free (keysym_name);
     }
 
-#ifdef USE_WAITFORKEY_CURSOR
-  ungrab_rat();
-#endif
+  if (rat_grabbed)
+    ungrab_rat();
 }
 
 static void
@@ -460,7 +445,7 @@ key_press (XEvent *ev)
   modifier = ev->xkey.state;
   cook_keycode ( &ev->xkey, &ks, &modifier, NULL, 0, 1);
 
-  if (ks == prefix_key.sym && (modifier == prefix_key.state))
+  if (ks == prefix_key.sym && (x11_mask_to_rp_mask (modifier) == prefix_key.state))
     {
       handle_key (s);
     }
@@ -600,11 +585,13 @@ property_notify (XEvent *ev)
 	case XA_WM_NORMAL_HINTS:
 	  PRINT_DEBUG ("updating window normal hints\n");
 	  update_normal_hints (win);
+	  if (win->state == NormalState)
+	    maximize (win);
 	  break;
 
 	case XA_WM_TRANSIENT_FOR:
 	  PRINT_DEBUG ("Transient for\n");
-	  win->transient = XGetTransientForHint (dpy, win->w, &win->transient_for);  
+	  win->transient = XGetTransientForHint (dpy, win->w, &win->transient_for);
 	  break;
 
 	default:
@@ -697,105 +684,87 @@ delegate_event (XEvent *ev)
   switch (ev->type)
     {
     case ConfigureRequest:
-      PRINT_DEBUG ("ConfigureRequest\n");
+      PRINT_DEBUG ("--- Handling ConfigureRequest ---\n");
       configure_request (&ev->xconfigurerequest);
       break;
-    case CirculateRequest:
-      PRINT_DEBUG ("CirculateRequest\n");
-      break;
+
     case CreateNotify:
-      PRINT_DEBUG ("CreateNotify\n");
+      PRINT_DEBUG ("--- Handling CreateNotify ---\n");
       new_window (&ev->xcreatewindow);
       break;
+
     case DestroyNotify:
-      PRINT_DEBUG ("DestroyNotify\n");
+      PRINT_DEBUG ("--- Handling DestroyNotify ---\n");
       destroy_window (&ev->xdestroywindow);
       break;
+
     case ClientMessage:
-      PRINT_DEBUG ("ClientMessage\n");
+      PRINT_DEBUG ("--- Handling ClientMessage ---\n");
       client_msg (&ev->xclient);
       break;
+
     case ColormapNotify:
-      PRINT_DEBUG ("ColormapNotify\n");
+      PRINT_DEBUG ("--- Handling ColormapNotify ---\n");
       colormap_notify (ev);
       break;
+
     case PropertyNotify:
-      PRINT_DEBUG ("PropertyNotify\n");
+      PRINT_DEBUG ("--- Handling PropertyNotify ---\n");
       property_notify (ev);
       break;
-    case SelectionClear:
-      PRINT_DEBUG ("SelectionClear\n");
-      break;
-    case SelectionNotify:
-      PRINT_DEBUG ("SelectionNotify\n");
-      break;
-    case SelectionRequest:
-      PRINT_DEBUG ("SelectionRequest\n");
-      break;
-    case EnterNotify:
-      PRINT_DEBUG ("EnterNotify\n");
-      break;
-    case ReparentNotify:
-      PRINT_DEBUG ("ReparentNotify\n");
-      break;
 
     case MapRequest:
-      PRINT_DEBUG ("MapRequest\n");
+      PRINT_DEBUG ("--- Handling MapRequest ---\n");
       map_request (ev);
       break;
 
     case KeyPress:
-      PRINT_DEBUG ("KeyPress %d %d\n", ev->xkey.keycode, ev->xkey.state);
+      PRINT_DEBUG ("--- Handling KeyPress ---\n");
       key_press (ev);
       break;
 
-    case KeyRelease: 
-      PRINT_DEBUG ("KeyRelease %d %d\n", ev->xkey.keycode, ev->xkey.state);
-      break;
-
     case UnmapNotify:
-      PRINT_DEBUG ("UnmapNotify\n");
+      PRINT_DEBUG ("--- Handling UnmapNotify ---\n");
       unmap_notify (ev);
       break;
 
-    case MotionNotify:
-      PRINT_DEBUG ("MotionNotify\n");
+    case FocusOut:
+      PRINT_DEBUG ("--- Handling FocusOut ---\n");
+      focus_change (&ev->xfocus);
       break;
 
-    case Expose:
-      PRINT_DEBUG ("Expose\n");
-      break;
-    case FocusOut:
-      PRINT_DEBUG ("FocusOut\n");
-      focus_change (&ev->xfocus);
-      break;
     case FocusIn:
-      PRINT_DEBUG ("FocusIn\n");
+      PRINT_DEBUG ("--- Handling FocusIn ---\n");
       focus_change (&ev->xfocus);
       break;
-    case ConfigureNotify:
-      PRINT_DEBUG ("ConfigureNotify\n");
-      configure_notify (&ev->xconfigure);
-      break;
-    case MapNotify:
-      PRINT_DEBUG ("MapNotify\n");
-      break;
+
     case MappingNotify:
-      PRINT_DEBUG ("MappingNotify\n");
+      PRINT_DEBUG ("--- Handling MappingNotify ---\n");
       mapping_notify( &ev->xmapping );
       break;
+
+    case ConfigureNotify:
+    case MapNotify:
+    case Expose:
+    case MotionNotify:
+    case KeyRelease:
+    case ReparentNotify:
+    case EnterNotify:
+    case SelectionRequest:
+    case SelectionNotify:
+    case SelectionClear:
+    case CirculateRequest:
+      /* Ignore these events. */
+      break;
+
     default:
-      PRINT_DEBUG ("Unhandled event %d\n", ev->type);
+      PRINT_DEBUG ("--- Unknown event %d ---\n",- ev->type);
     }
 }
 
 static void
-get_event (XEvent *ev)
+handle_signals ()
 {
-  int x_fd;
-  fd_set fds;
-  struct timeval t;
-
   /* An alarm means we need to hide the popup windows. */
   if (alarm_signalled > 0)
     {
@@ -803,64 +772,61 @@ get_event (XEvent *ev)
 
       PRINT_DEBUG ("Alarm recieved.\n");
 
-      for (i=0; i<num_screens; i++)
-	{
+      /* Only hide the bar if it times out. */
+      if (defaults.bar_timeout > 0)
+	for (i=0; i<num_screens; i++)
 	  hide_bar (&screens[i]);
-	}
+
       hide_frame_indicator();
       alarm_signalled = 0;
     }
 
   if (hup_signalled > 0)
     {
-      PRINT_DEBUG ("Restarting with a fresh plate.\n"); 
-      send_restart ();
+      clean_up (); 
+      execvp(myargv[0], myargv);
     }
 
   if (kill_signalled > 0)
     {
-      fprintf (stderr, "ratpoison: Agg! I've been SHOT!\n"); 
+      PRINT_DEBUG ("Exiting\n");
       clean_up ();
-      exit (EXIT_FAILURE);
+      exit (EXIT_SUCCESS);
     }
+}
 
-  /* Is there anything in the event qeue? */
-  if (QLength (dpy) > 0)
-    {
-      XNextEvent (dpy, ev);
-      return;
-    }
+/* The main loop. */
+void
+listen_for_events ()
+{
+  int x_fd;
+  fd_set fds;
 
-  /* If the event queue is empty, then select on it until there is
-     something. */
   x_fd = ConnectionNumber (dpy);
   FD_ZERO (&fds);
-  FD_SET (x_fd, &fds);
-  t.tv_sec = 0;
-  t.tv_usec = 0;
 
-  if (select(x_fd+1, &fds, NULL, NULL, &t) == 1) 
-    {
-    XNextEvent(dpy, ev);
-    return;
-    }
-
-  XFlush(dpy);
-  FD_SET(x_fd, &fds);
-
-  if (select(x_fd+1, &fds, NULL, NULL, NULL) == 1) 
-    {
-    XNextEvent(dpy, ev);
-    return;
-    }
-}
-
-void
-handle_events ()
-{
+  /* Loop forever. */
   for (;;) 
     {
-      get_event (&rp_current_event);
-      delegate_event (&rp_current_event);
+      handle_signals ();
+
+      /* Report any X11 errors that have occurred. */
+      if (rp_error_msg)
+	{
+	  marked_message_printf (0, 6, "ERROR: %s", rp_error_msg);
+	  free (rp_error_msg);
+	  rp_error_msg = NULL;
+	}
+
+      /* Handle the next event. */
+      FD_SET (x_fd, &fds);
+      XFlush(dpy);
+      
+      if (QLength (dpy) > 0
+	  || select(x_fd+1, &fds, NULL, NULL, NULL) == 1)
+	{
+	  XNextEvent (dpy, &rp_current_event);
+	  delegate_event (&rp_current_event);
+	}
     }
 }
