@@ -28,6 +28,53 @@
 
 #include "ratpoison.h"
 
+/* Variables to keep track of input history. */
+static char *input_history[INPUT_MAX_HISTORY];
+static int input_num_history_entries = 0;
+
+/* Convert an X11 modifier mask to the rp modifier mask equivalent, as
+   best it can (the X server may not have a hyper key defined, for
+   instance). */
+unsigned int
+x11_mask_to_rp_mask (unsigned int mask)
+{
+  unsigned int result = 0;
+
+  PRINT_DEBUG ("x11 mask = %x\n", mask);
+
+  result |= mask & ControlMask ? RP_CONTROL_MASK:0;
+  result |= mask & rp_modifier_info.meta_mod_mask ? RP_META_MASK:0;
+  result |= mask & rp_modifier_info.alt_mod_mask ? RP_ALT_MASK:0;
+  result |= mask & rp_modifier_info.hyper_mod_mask ? RP_HYPER_MASK:0;
+  result |= mask & rp_modifier_info.super_mod_mask ? RP_SUPER_MASK:0;
+
+  PRINT_DEBUG ("rp mask = %x\n", mask);
+
+  return result;
+}
+
+/* Convert an rp modifier mask to the x11 modifier mask equivalent, as
+   best it can (the X server may not have a hyper key defined, for
+   instance). */
+unsigned int
+rp_mask_to_x11_mask (unsigned int mask)
+{
+  unsigned int result = 0;
+
+  PRINT_DEBUG ("rp mask = %x\n", mask);
+
+  result |= mask & RP_CONTROL_MASK ? ControlMask:0;
+  result |= mask & RP_META_MASK ? rp_modifier_info.meta_mod_mask:0;
+  result |= mask & RP_ALT_MASK ? rp_modifier_info.alt_mod_mask:0;
+  result |= mask & RP_HYPER_MASK ? rp_modifier_info.hyper_mod_mask:0;
+  result |= mask & RP_SUPER_MASK ? rp_modifier_info.super_mod_mask:0;
+
+  PRINT_DEBUG ("x11 mask = %x\n", mask);
+
+  return result;
+}
+
+
 /* Figure out what keysyms are attached to what modifiers */
 void
 update_modifier_map ()
@@ -125,6 +172,9 @@ grab_key (int keycode, unsigned int modifiers, Window grab_window)
   unsigned int mod_list[8];
   int i;
 
+  /* Convert to a modifier mask that X Windows will understand. */
+  modifiers = rp_mask_to_x11_mask (modifiers);
+
   /* Create a list of all possible combinations of ignored
      modifiers. Assumes there are only 3 ignored modifiers. */
   mod_list[0] = 0;
@@ -149,18 +199,25 @@ grab_key (int keycode, unsigned int modifiers, Window grab_window)
 char *
 keysym_to_string (KeySym keysym, unsigned int modifier)
 {
+  static char *null_string = "NULL"; /* A NULL string. */
   struct sbuf *name;
   char *tmp;
 
   name = sbuf_new (0);
 
-  if (modifier & ControlMask) sbuf_concat (name, "C-");
-  if (modifier & rp_modifier_info.meta_mod_mask) sbuf_concat (name, "M-");
-  if (modifier & rp_modifier_info.alt_mod_mask) sbuf_concat (name, "A-");
-  if (modifier & rp_modifier_info.hyper_mod_mask) sbuf_concat (name, "H-");
-  if (modifier & rp_modifier_info.super_mod_mask) sbuf_concat (name, "S-");
+  if (modifier & RP_CONTROL_MASK) sbuf_concat (name, "C-");
+  if (modifier & RP_META_MASK) sbuf_concat (name, "M-");
+  if (modifier & RP_ALT_MASK) sbuf_concat (name, "A-");
+  if (modifier & RP_HYPER_MASK) sbuf_concat (name, "H-");
+  if (modifier & RP_SUPER_MASK) sbuf_concat (name, "S-");
     
-  sbuf_concat (name, XKeysymToString (keysym));
+  /* On solaris machines (perhaps other machines as well) this call
+     can return NULL. In this case use the "NULL" string. */
+  tmp = XKeysymToString (keysym);
+  if (tmp == NULL)
+    tmp = null_string;
+
+  sbuf_concat (name, tmp);
 
   /* Eat the nut and throw away the shells. */
   tmp = sbuf_get (name);
@@ -200,71 +257,6 @@ cook_keycode (XKeyEvent *ev, KeySym *keysym, unsigned int *mod, char *keysym_nam
   return nbytes;
 }
 
-/* void */
-/* cook_keycode (KeyCode keycode, KeySym *keysym, unsigned int *mod) */
-/* { */
-/*   KeySym normal, shifted; */
-
-/*   /\* FIXME: Although this should theoretically work, the mod that */
-/*      mode_switch is on doesn't seem to get activated. Instead the */
-/*      2<<13 bit gets set! It doesn't seem to matter which mod I put */
-/*      Mode_switch on. So if this doesn't work try uncommented the line */
-/*      below and commented the current one. *\/ */
-
-/* /\*   if (*mod & 8192) *\/ */
-/*   if (*mod & rp_modifier_info.mode_switch_mask) */
-/*     { */
-/*       normal = XKeycodeToKeysym(dpy, keycode, 2); */
-/*       if (normal == NoSymbol) normal = XKeycodeToKeysym(dpy, keycode, 0); */
-/*       shifted = XKeycodeToKeysym(dpy, keycode, 3); */
-/*       if (shifted == NoSymbol) shifted = XKeycodeToKeysym(dpy, keycode, 1); */
-
-/*       /\* Remove the mode switch modifier since we have dealt with it. *\/ */
-/*       *mod &= ~rp_modifier_info.mode_switch_mask; */
-/*     } */
-/*   else */
-/*     { */
-/*       normal = XKeycodeToKeysym(dpy, keycode, 0); */
-/*       shifted = XKeycodeToKeysym(dpy, keycode, 1); */
-/*     } */
-
-/*   /\* FIXME: eew, this looks gross. *\/ */
-/*   if (*mod & (ShiftMask | LockMask)) */
-/*     { */
-/*       /\* if the shifted code is not defined, then we use the normal */
-/* 	 keysym and keep the shift mask *\/ */
-/*       if (shifted == NoSymbol) */
-/* 	{ */
-/* 	  *keysym = normal; */
-/* 	} */
-/*       /\* But if the shifted code is defined, we use it and remove the */
-/* 	 shift mask *\/ */
-/*       else if (*mod & ShiftMask) */
-/* 	{ */
-/* 	  *keysym = shifted; */
-/* 	  *mod &= ~(ShiftMask | LockMask); */
-/* 	} */
-/*       /\* If caps lock is on, use shifted for alpha keys *\/ */
-/*       else if (normal >= XK_a  */
-/* 	       && normal <= XK_z */
-/* 	       && *mod & LockMask) */
-/* 	{ */
-/* 	  *keysym = shifted; */
-/* 	} */
-/*       else */
-/* 	{ */
-/* 	  *keysym = normal; */
-/* 	} */
-/*     } */
-/*   else */
-/*     { */
-/*       *keysym = normal; */
-/*     } */
-
-/*   PRINT_DEBUG ("cooked keysym: %ld '%c' mask: %d\n",  */
-/* 	       *keysym, (char)*keysym, *mod); */
-/* } */
-
 int
 read_key (KeySym *keysym, unsigned int *modifiers, char *keysym_name, int len)
 {
@@ -284,32 +276,40 @@ read_key (KeySym *keysym, unsigned int *modifiers, char *keysym_name, int len)
 static void
 update_input_window (screen_info *s, char *prompt, char *input, int input_len)
 {
-  int 	prompt_width = XTextWidth (s->font, prompt, strlen (prompt));
-  int 	input_width  = XTextWidth (s->font, input, input_len);
+  int 	prompt_width = XTextWidth (defaults.font, prompt, strlen (prompt));
+  int 	input_width  = XTextWidth (defaults.font, input, input_len);
   int 	width;
 
-  width = BAR_X_PADDING * 2 + prompt_width + input_width;
+  width = defaults.bar_x_padding * 2 + prompt_width + input_width;
 
-  if (width < INPUT_WINDOW_SIZE + prompt_width)
+  if (width < defaults.input_window_size + prompt_width)
     {
-      width = INPUT_WINDOW_SIZE + prompt_width;
+      width = defaults.input_window_size + prompt_width;
     }
 
   XMoveResizeWindow (dpy, s->input_window, 
 		     bar_x (s, width), bar_y (s), width,
-		     (FONT_HEIGHT (s->font) + BAR_Y_PADDING * 2));
+		     (FONT_HEIGHT (defaults.font) + defaults.bar_y_padding * 2));
 
   XClearWindow (dpy, s->input_window);
+  XSync (dpy, False);
 
   XDrawString (dpy, s->input_window, s->normal_gc, 
-	       BAR_X_PADDING, 
-	       BAR_Y_PADDING + s->font->max_bounds.ascent, prompt, 
+	       defaults.bar_x_padding, 
+	       defaults.bar_y_padding + defaults.font->max_bounds.ascent, prompt, 
 	       strlen (prompt));
 
   XDrawString (dpy, s->input_window, s->normal_gc, 
-	       BAR_X_PADDING + prompt_width,
-	       BAR_Y_PADDING + s->font->max_bounds.ascent, input, 
+	       defaults.bar_x_padding + prompt_width,
+	       defaults.bar_y_padding + defaults.font->max_bounds.ascent, input, 
 	       input_len);
+
+  /* Draw a cheap-o cursor. */
+  XDrawLine (dpy, s->input_window, s->normal_gc, 
+	     defaults.bar_x_padding + prompt_width + input_width + 2, 
+	     defaults.bar_y_padding + 1, 
+	     defaults.bar_x_padding + prompt_width + input_width + 2,
+	     defaults.bar_y_padding + FONT_HEIGHT (defaults.font) - 1);
 }
 
 char *
@@ -333,8 +333,9 @@ get_more_input (char *prompt, char *preinput)
   int revert;
   Window fwin;
   char *str;
+  int history_index = input_num_history_entries;
 
-  /* Allocate some memory to start with */
+  /* Allocate some memory to start with. */
   str = (char *) xmalloc ( allocated_len );
 
   /* load in the preinput */
@@ -345,8 +346,9 @@ get_more_input (char *prompt, char *preinput)
   hide_bar (s);
 
   XMapWindow (dpy, s->input_window);
-  XClearWindow (dpy, s->input_window);
   XRaiseWindow (dpy, s->input_window);
+  XClearWindow (dpy, s->input_window);
+  XSync (dpy, False);
 
   update_input_window (s, prompt, str, cur_len);
 
@@ -364,6 +366,46 @@ get_more_input (char *prompt, char *preinput)
 	  if (cur_len > 0) cur_len--;
 	  update_input_window(s, prompt, str, cur_len);
 	}
+      else if (ch == INPUT_PREV_HISTORY_KEY 
+	       && modifier == INPUT_PREV_HISTORY_MODIFIER)
+	{
+	  /* Cycle through the history. */
+	  if (input_num_history_entries > 0)
+	    {
+	      history_index--;
+	      if (history_index < 0)
+		{
+		  history_index = input_num_history_entries - 1;
+		}
+
+	      free (str);
+	      str = xstrdup (input_history[history_index]);
+	      allocated_len = strlen (str) + 1;
+	      cur_len = allocated_len - 1;
+
+	      update_input_window (s, prompt, str, cur_len);
+	    }
+	}
+      else if (ch == INPUT_NEXT_HISTORY_KEY 
+	       && modifier == INPUT_NEXT_HISTORY_MODIFIER)
+	{
+	  /* Cycle through the history. */
+	  if (input_num_history_entries > 0)
+	    {
+	      history_index++;
+	      if (history_index >= input_num_history_entries)
+		{
+		  history_index = 0;
+		}
+
+	      free (str);
+	      str = xstrdup (input_history[history_index]);
+	      allocated_len = strlen (str) + 1;
+	      cur_len = allocated_len - 1;
+
+	      update_input_window (s, prompt, str, cur_len);
+	    }
+	}
       else if (ch == INPUT_ABORT_KEY && modifier == INPUT_ABORT_MODIFIER)
 	{
 	  /* User aborted. */
@@ -391,6 +433,24 @@ get_more_input (char *prompt, char *preinput)
     }
 
   str[cur_len] = 0;
+
+  /* Push the history entries down. */
+  if (input_num_history_entries >= INPUT_MAX_HISTORY)
+    {
+      int i;
+      free (input_history[0]);
+      for (i=0; i<INPUT_MAX_HISTORY-1; i++)
+	{
+	  input_history[i] = input_history[i+1];
+	}
+
+      input_num_history_entries--;
+    }
+
+  /* Store the string in the history. */
+  input_history[input_num_history_entries] = xstrdup (str);
+  input_num_history_entries++;
+
   XSetInputFocus (dpy, fwin, RevertToPointerRoot, CurrentTime);
   XUnmapWindow (dpy, s->input_window);
   return str;
