@@ -52,24 +52,21 @@ Atom wm_delete;
 Atom wm_take_focus;
 Atom wm_colormaps;
 
-Atom rp_restart;
-Atom rp_kill;
 Atom rp_command;
 Atom rp_command_request;
 Atom rp_command_result;
 
+int rp_current_screen;
 screen_info *screens;
 int num_screens;
 Display *dpy;
 
+struct rp_defaults defaults;
+
 int ignore_badwindow = 0;
 
-static XFontStruct *font;
-
 char **myargv;
 
-XGCValues gv;
-
 struct rp_key prefix_key;
 
 struct modifier_info rp_modifier_info;
@@ -80,16 +77,16 @@ int rp_honour_normal_raise = 1;
 int rp_honour_transient_map = 1;
 int rp_honour_normal_map = 1;
 
+char *rp_error_msg = NULL;
+
 /* Command line options */
 static struct option ratpoison_longopts[] = 
   { {"help", 	no_argument, 		0, 	'h'},
     {"version", no_argument, 		0, 	'v'},
-    {"restart", no_argument, 		0, 	'r'},
-    {"kill", 	no_argument, 		0, 	'k'},
     {"command", required_argument, 	0, 	'c'},
     {0, 	0, 			0, 	0} };
 
-static char ratpoison_opts[] = "hvrkc:";
+static char ratpoison_opts[] = "hvc:";
 
 void
 fatal (const char *msg)
@@ -117,6 +114,75 @@ xrealloc (void *ptr, size_t size)
   return value;
 }
 
+char *
+xstrdup (char *s)
+{
+  char *value;
+  value = strdup (s);
+  if (value == 0)
+    fatal ("Virtual memory exhausted");
+  return value;
+}
+
+/* Return a new string based on fmt. */
+char *
+xvsprintf (char *fmt, va_list ap)
+{
+  int size, nchars;
+  char *buffer;
+  va_list ap_copy;
+
+  /* A resonable starting value. */
+  size = strlen (fmt) + 1;
+  buffer = (char *)xmalloc (size);
+
+  while (1)
+    {
+#if defined(va_copy)
+      va_copy (ap_copy, ap);
+#elif defined(__va_copy)
+      __va_copy (ap_copy, ap);
+#else
+      /* If there is no copy macro then this MAY work. On some systems
+	 this could fail because va_list is a pointer so assigning one
+	 to the other as below wouldn't make a copy of the data, but
+	 just the pointer to the data. */
+      ap_copy = ap;
+#endif
+      nchars = vsnprintf (buffer, size, fmt, ap_copy);
+#if defined(va_copy) || defined(__va_copy)
+      va_end (ap_copy);
+#endif
+
+      if (nchars > -1 && nchars < size)
+	return buffer;
+      else if (nchars > -1)
+	size = nchars + 1;
+      else
+	size *= 2;
+
+      /* Resize the buffer and try again. */
+      buffer = (char *)xrealloc (buffer, size);
+    }
+
+  /* Never gets here. */
+  return NULL;
+}
+
+/* Return a new string based on fmt. */
+char *
+xsprintf (char *fmt, ...)
+{
+  char *buffer;
+  va_list ap;
+
+  va_start (ap, fmt);
+  buffer = xvsprintf (fmt, ap);
+  va_end (ap);
+
+  return buffer;
+}
+
 void
 sighandler (int signum)
 {
@@ -147,14 +213,15 @@ handler (Display *d, XErrorEvent *e)
 
   if (ignore_badwindow && e->error_code == BadWindow) return 0;
 
-  strcpy (error_msg, "ERROR: ");
+  XGetErrorText (d, e->error_code, error_msg, sizeof (error_msg));
+  fprintf (stderr, "ratpoison: ERROR: %s!\n", error_msg);
 
-  XGetErrorText (d, e->error_code, error_msg + 7, sizeof (error_msg) - 7);
-  fprintf (stderr, "ratpoison: %s!\n", error_msg);
+  /* If there is already an error to report, replace it with this new
+     one. */
+  if (rp_error_msg) 
+    free (rp_error_msg);
+  rp_error_msg = xstrdup (error_msg);
 
-  marked_message (error_msg, 0, strlen (error_msg));
-
-/*   exit (EXIT_FAILURE);  */
   return 0;
 }
 
@@ -203,8 +270,6 @@ print_help ()
   printf ("Help for %s %s\n\n", PACKAGE, VERSION);
   printf ("-h, --help            Display this help screen\n");
   printf ("-v, --version         Display the version\n");
-  printf ("-r, --restart         Restart ratpoison\n");
-  printf ("-k, --kill            Kill ratpoison\n");
   printf ("-c, --command         Send ratpoison a colon-command\n\n");
 
   printf ("Report bugs to ratpoison-devel@lists.sourceforge.net\n\n");
@@ -345,15 +410,50 @@ show_welcome_message ()
   free (prefix);
 }
 
+static void
+init_defaults ()
+{
+  defaults.win_gravity     = NorthWestGravity;
+  defaults.trans_gravity   = CenterGravity;
+  defaults.maxsize_gravity = CenterGravity;
+
+  defaults.input_window_size   = 200;
+  defaults.window_border_width = 1;
+  defaults.bar_x_padding       = 0;
+  defaults.bar_y_padding       = 0;
+  defaults.bar_location        = NorthEastGravity;
+  defaults.bar_timeout 	       = 5;
+  defaults.bar_border_width    = 1;
+
+  defaults.frame_indicator_timeout = 1;
+
+  defaults.padding_left   = 0;
+  defaults.padding_right  = 0;
+  defaults.padding_top 	  = 0;
+  defaults.padding_bottom = 0;
+
+  defaults.font = XLoadQueryFont (dpy, "9x15bold");
+  if (defaults.font == NULL)
+    {
+      fprintf (stderr, "ratpoison: Cannot load font %s.\n", "9x15bold");
+      exit (EXIT_FAILURE);
+    }
+
+  defaults.wait_for_key_cursor = 1;
+
+  defaults.window_fmt = xstrdup ("%n%s%t");
+
+  defaults.win_name = 0;
+  defaults.startup_message = 1;
+}
+
 int
 main (int argc, char *argv[])
 {
   int i;
   int c;
-  int do_kill = 0;
-  int do_restart = 0;
-  int do_command = 0;
-  char *command = NULL;
+  char **command = NULL;
+  int cmd_count = 0;
 
   myargv = argv;
 
@@ -373,16 +473,19 @@ main (int argc, char *argv[])
 	case 'v':
 	  print_version ();
 	  break;
-	case 'k':
-	  do_kill = 1;
-	  break;
-	case 'r':
-	  do_restart = 1;
-	  break;
 	case 'c':
-	  command = xmalloc (strlen (optarg) + 1);
-	  strcpy (command, optarg);
-	  do_command = 1;
+	  if (!command)
+	    {
+	      command = xmalloc (sizeof(char *));
+	      cmd_count = 0;
+	    }
+	  else
+	    {
+	      command = xrealloc (command, sizeof (char *) * (cmd_count + 1));
+	    }
+
+	  command[cmd_count] = xstrdup (optarg);
+	  cmd_count++;
 	  break;
 	default:
 	  exit (EXIT_FAILURE);
@@ -396,29 +499,20 @@ main (int argc, char *argv[])
     }
 
   /* Set ratpoison specific Atoms. */
-  rp_restart = XInternAtom (dpy, "RP_RESTART", False);
-  rp_kill = XInternAtom (dpy, "RP_KILL", False);
   rp_command = XInternAtom (dpy, "RP_COMMAND", False);
   rp_command_request = XInternAtom (dpy, "RP_COMMAND_REQUEST", False);
   rp_command_result = XInternAtom (dpy, "RP_COMMAND_RESULT", False);
 
-  if (do_kill)
+  if (cmd_count > 0)
     {
-      send_kill ();
-      XSync (dpy, False);
-      XCloseDisplay (dpy);
-      return EXIT_SUCCESS;
-    }
-  if (do_restart)
-    {
-      send_restart ();
-      XSync (dpy, False);
-      XCloseDisplay (dpy);
-      return EXIT_SUCCESS;
-    }
-  if (do_command)
-    {
-      send_command (command);
+      int i;
+
+      for (i=0; i<cmd_count; i++)
+	{
+	  send_command (command[i]);
+	  free (command[i]);
+	}
+
       free (command);
       XCloseDisplay (dpy);
       return EXIT_SUCCESS;
@@ -440,36 +534,43 @@ main (int argc, char *argv[])
   set_sig_handler (SIGHUP, hup_handler);
 
   /* Setup ratpoison's internal structures */
+  init_defaults();
   init_numbers ();
-  init_window_list ();
-  init_frame_list ();
-  update_modifier_map ();
-  initialize_default_keybindings ();
-
-  font = XLoadQueryFont (dpy, FONT);
-  if (font == NULL)
-    {
-      fprintf (stderr, "ratpoison: Cannot load font %s.\n", FONT);
-      exit (EXIT_FAILURE);
-    }
-
-  num_screens = ScreenCount (dpy);
-  screens = (screen_info *)xmalloc (sizeof (screen_info) * num_screens);
-
-  PRINT_DEBUG ("%d screens.\n", num_screens);
 
   /* Initialize the screens */
+  num_screens = ScreenCount (dpy);
+  screens = (screen_info *)xmalloc (sizeof (screen_info) * num_screens);
+  PRINT_DEBUG ("%d screens.\n", num_screens);
+
   for (i=0; i<num_screens; i++)
     {
       init_screen (&screens[i], i);
     }
 
+  init_window_list ();
+  init_frame_lists ();
+  update_modifier_map ();
+  initialize_default_keybindings ();
+
+  /* Scan for windows */
+  rp_current_screen = 0;
+  for (i=0; i<num_screens; i++)
+    {
+      scanwins (&screens[i]);
+    }
+
   read_startup_files ();
 
   /* Indicate to the user that ratpoison has booted. */
-  show_welcome_message();
-  
-  handle_events ();
+  if (defaults.startup_message)
+    show_welcome_message();
+
+  /* If no window has focus, give the key_window focus. */
+  if (current_window() == NULL)
+    XSetInputFocus (dpy, current_screen()->key_window, 
+		    RevertToPointerRoot, CurrentTime);
+
+  listen_for_events ();
 
   return EXIT_SUCCESS;
 }
@@ -483,43 +584,45 @@ init_rat_cursor (screen_info *s)
 static void
 init_screen (screen_info *s, int screen_num)
 {
-  XColor fg_color, bg_color,/*  bold_color, */ junk;
+  XGCValues gv;
 
   /* Select on some events on the root window, if this fails, then
      there is already a WM running and the X Error handler will catch
      it, terminating ratpoison. */
   XSelectInput(dpy, RootWindow (dpy, screen_num),
                PropertyChangeMask | ColormapChangeMask
-               | SubstructureRedirectMask | KeyPressMask | KeyReleaseMask
-               | SubstructureNotifyMask );
+               | SubstructureRedirectMask | SubstructureNotifyMask );
   XSync (dpy, False);
 
+  /* Build the display string for each screen */
+  s->display_string = xmalloc (strlen(DisplayString (dpy)) + 21);
+  sprintf (s->display_string, "DISPLAY=%s", DisplayString (dpy));
+  if (strrchr (DisplayString (dpy), ':'))
+    {
+      char *dot;
+
+      dot = strrchr(s->display_string, '.');
+      if (dot)
+	sprintf(dot, ".%i", screen_num);
+    }
+
   s->screen_num = screen_num;
   s->root = RootWindow (dpy, screen_num);
   s->def_cmap = DefaultColormap (dpy, screen_num);
-  s->font = font;
   XGetWindowAttributes (dpy, s->root, &s->root_attr);
   
   init_rat_cursor (s);
 
-  /* Get our program bar colors */
-  if (!XAllocNamedColor (dpy, s->def_cmap, FOREGROUND, &fg_color, &junk))
-    {
-      fprintf (stderr, "ratpoison: Unknown color '%s'\n", FOREGROUND);
-    }
-
-  if (!XAllocNamedColor (dpy, s->def_cmap, BACKGROUND, &bg_color, &junk))
-    {
-      fprintf (stderr, "ratpoison: Unknown color '%s'\n", BACKGROUND);
-    }
+  s->fg_color = BlackPixel (dpy, s->screen_num);
+  s->bg_color = WhitePixel (dpy, s->screen_num);
 
   /* Setup the GC for drawing the font. */
-  gv.foreground = fg_color.pixel;
-  gv.background = bg_color.pixel;
+  gv.foreground = s->fg_color;
+  gv.background = s->bg_color;
   gv.function = GXcopy;
   gv.line_width = 1;
   gv.subwindow_mode = IncludeInferiors;
-  gv.font = font->fid;
+  gv.font = defaults.font->fid;
   s->normal_gc = XCreateGC(dpy, s->root, 
 			   GCForeground | GCBackground | GCFunction 
 			   | GCLineWidth | GCSubwindowMode | GCFont, 
@@ -527,55 +630,41 @@ init_screen (screen_info *s, int screen_num)
 
   /* Create the program bar window. */
   s->bar_is_raised = 0;
-  s->bar_window = XCreateSimpleWindow (dpy, s->root, 0, 0,
-				       1, 1, 1, fg_color.pixel, bg_color.pixel);
-  XSelectInput (dpy, s->bar_window, StructureNotifyMask);
+  s->bar_window = XCreateSimpleWindow (dpy, s->root, 0, 0, 1, 1, 
+				       defaults.bar_border_width, 
+				       s->fg_color, s->bg_color);
 
   /* Setup the window that will recieve all keystrokes once the prefix
      key has been pressed. */
-  s->key_window = XCreateSimpleWindow (dpy, s->root, 0, 0, 1, 1, 0, WhitePixel (dpy, 0), BlackPixel (dpy, 0));
+  s->key_window = XCreateSimpleWindow (dpy, s->root, 0, 0, 1, 1, 0, 
+				       WhitePixel (dpy, s->screen_num), 
+				       BlackPixel (dpy, s->screen_num));
   XSelectInput (dpy, s->key_window, KeyPressMask );
   XMapWindow (dpy, s->key_window);
 
   /* Create the input window. */
-  s->input_window = XCreateSimpleWindow (dpy, s->root, 0, 0, 
-  					 1, 1, 1, fg_color.pixel, bg_color.pixel);
+  s->input_window = XCreateSimpleWindow (dpy, s->root, 0, 0, 1, 1, 
+					 defaults.bar_border_width, 
+					 s->fg_color, s->bg_color);
   XSelectInput (dpy, s->input_window, KeyPressMask );
 
   /* Create the frame indicator window */
-  s->frame_window = XCreateSimpleWindow (dpy, s->root, 1, 1, 1, 1, 1, 
-					 fg_color.pixel, bg_color.pixel);
-  XSelectInput (dpy, s->frame_window, KeyPressMask );
+  s->frame_window = XCreateSimpleWindow (dpy, s->root, 1, 1, 1, 1, defaults.bar_border_width, 
+					 s->fg_color, s->bg_color);
 
   /* Create the help window */
   s->help_window = XCreateSimpleWindow (dpy, s->root, 0, 0, s->root_attr.width,
-					s->root_attr.height, 1, fg_color.pixel, bg_color.pixel);
+					s->root_attr.height, 0, s->fg_color, s->bg_color);
   XSelectInput (dpy, s->help_window, KeyPressMask);
 
   XSync (dpy, 0);
-
-  scanwins (s);
 }
 
 void
 clean_up ()
 {
-  rp_window *cur;
   int i;
 
-  /* Map any iconized windows. */
-  for (cur = rp_mapped_window_sentinel->next;
-       cur != rp_mapped_window_sentinel;
-       cur = cur->next)
-    {
-      if (cur->state == IconicState)
-	{
-	  unhide_window (cur);
-	}
-    }
-  
-  XSync (dpy, False);
-
   for (i=0; i<num_screens; i++)
     {
       XDestroyWindow (dpy, screens[i].bar_window);
@@ -589,7 +678,7 @@ clean_up ()
       XFreeGC (dpy, screens[i].normal_gc);
     }
 
-  XFreeFont (dpy, font);
+  XFreeFont (dpy, defaults.font);
 
   XSetInputFocus (dpy, PointerRoot, RevertToPointerRoot, CurrentTime);
   XCloseDisplay (dpy);
