@@ -64,10 +64,7 @@ ungrab_prefix_key (Window w)
 screen_info*
 current_screen ()
 {
-  if (current_window())
-    return current_window()->scr;
-  else
-    return &screens[0];
+  return &screens[rp_current_screen];
 }
 
 void
@@ -77,15 +74,23 @@ update_normal_hints (rp_window *win)
 
   XGetWMNormalHints (dpy, win->w, win->hints, &supplied);
 
-  PRINT_DEBUG ("hints: minx: %d miny: %d maxx: %d maxy: %d incx: %d incy: %d\n", 
-	       win->hints->min_width, win->hints->min_height,
-	       win->hints->max_width, win->hints->max_height,
-	       win->hints->width_inc, win->hints->height_inc);
+  /* Print debugging output for window hints. */
+#ifdef DEBUG
+  if (win->hints->flags & PMinSize)
+    PRINT_DEBUG ("minx: %d miny: %d\n", win->hints->min_width, win->hints->min_height);
+
+  if (win->hints->flags & PMaxSize)
+    PRINT_DEBUG ("maxx: %d maxy: %d\n", win->hints->max_width, win->hints->max_height);
+
+  if (win->hints->flags & PResizeInc)
+    PRINT_DEBUG ("incx: %d incy: %d\n", win->hints->width_inc, win->hints->height_inc);
+
+#endif
 }
 		     
 
-char *
-get_window_name (Window w)
+static char *
+get_wmname (Window w)
 {
   char *name;
   XTextProperty text;
@@ -107,19 +112,85 @@ get_window_name (Window w)
   if (list_len > 0)
     {
       name = xmalloc (strlen (name_list[0]) + 1);
-
       strcpy (name, name_list[0]);
-
-      /* Its our responsibility to free this. */ 
-      XFreeStringList (name_list);
-
-      return name;
+    }
+  else
+    {
+      name = NULL;
     }
 
   /* Its our responsibility to free this. */ 
   XFreeStringList (name_list);
 
-  return NULL;
+  return name;
+}
+
+static XClassHint *
+get_class_hints (Window w)
+{
+  XClassHint *class;
+
+  class = XAllocClassHint();
+
+  if (class == NULL)
+    {
+      PRINT_ERROR ("Not enough memory for WM_CLASS structure.\n");
+      exit (EXIT_FAILURE);
+    }
+
+  XGetClassHint (dpy, w, class);
+
+  return class;
+}
+
+static char *
+get_res_name (Window w)
+{
+  XClassHint *class;
+  char *name;
+
+  class = get_class_hints (w);
+
+  if (class->res_name)
+    {
+      name = (char *)xmalloc (strlen (class->res_name) + 1);
+      strcpy (name, class->res_name);
+    }
+  else
+    {
+      name = NULL;
+    }
+
+  XFree (class->res_name);
+  XFree (class->res_class);
+  XFree (class);
+
+  return name;
+}
+
+static char *
+get_res_class (Window w)
+{
+  XClassHint *class;
+  char *name;
+
+  class = get_class_hints (w);
+
+  if (class->res_class)
+    {
+      name = (char *)xmalloc (strlen (class->res_class) + 1);
+      strcpy (name, class->res_class);
+    }
+  else
+    {
+      name = NULL;
+    }
+
+  XFree (class->res_name);
+  XFree (class->res_class);
+  XFree (class);
+
+  return name;
 }
 
 /* Reget the WM_NAME property for the window and update its name. */
@@ -127,46 +198,52 @@ int
 update_window_name (rp_window *win)
 {
   char *newstr;
-  char *loc;
 
   /* Don't overwrite the window name if the user specified one. */
-  if (win->named) return 0;
+/*   if (win->named) return 0; */
 
-  newstr = get_window_name (win->w);
+  newstr = get_wmname (win->w);
   if (newstr != NULL)
     {
-      free (win->name);
-      win->name = newstr;
+      free (win->wm_name);
+      win->wm_name = newstr;
     }
 
-  /* A bit of a hack. If there's a : in the string, crop the
-     string off there. This is mostly brought on by netscape's
-     disgusting tendency to put its current URL in the WMName!!
-     arg! */
-  loc = strchr (win->name, ':');
-  if (loc) loc[0] = '\0';
+  newstr = get_res_class (win->w);
+  if (newstr != NULL)
+    {
+      free (win->res_class);
+      win->res_class = newstr;
+    }
+
+  newstr = get_res_name (win->w);
+  if (newstr != NULL)
+    {
+      free (win->res_name);
+      win->res_name = newstr;
+    }
 
   return 1;
 }
 
 /* Send an artificial configure event to the window. */ 
 void
-send_configure (rp_window *win)
+send_configure (Window w, int x, int y, int width, int height, int border)
 {
   XConfigureEvent ce;
 
   ce.type = ConfigureNotify;
-  ce.event = win->w;
-  ce.window = win->w;
-  ce.x = win->x;
-  ce.y = win->y;
-  ce.width = win->width;
-  ce.height = win->height;
-  ce.border_width = win->border;
+  ce.event = w;
+  ce.window = w;
+  ce.x = x;
+  ce.y = y;
+  ce.width = width;
+  ce.height = height;
+  ce.border_width = border;
   ce.above = None;
   ce.override_redirect = 0;
 
-  XSendEvent (dpy, win->w, False, StructureNotifyMask, (XEvent*)&ce);
+  XSendEvent (dpy, w, False, StructureNotifyMask, (XEvent*)&ce);
 }
 
 void
@@ -190,6 +267,8 @@ update_window_information (rp_window *win)
 
   /* Transient status */
   win->transient = XGetTransientForHint (dpy, win->w, &win->transient_for);  
+
+  update_window_gravity (win);
 }
 
 void
@@ -212,7 +291,7 @@ unmanage (rp_window *w)
 	 shortly after another window is mapped most likely by the
 	 same app. */
 
-      send_kill();
+      kill_signalled = 1;
     }
 #endif
 }
@@ -243,11 +322,12 @@ scanwins(screen_info *s)
       win = add_to_window_list (s, wins[i]);
 
       PRINT_DEBUG ("map_state: %d\n", attr.map_state);
-      if (attr.map_state == IsViewable) 
-	{
-	  win->state = NormalState;
-	  map_window (win);
-	}
+      
+      /* Collect mapped and iconized windows. */
+      if (attr.map_state == IsViewable
+	  || (attr.map_state == IsUnmapped 
+	      && get_state (win) == IconicState))
+	map_window (win);
     }
 
   XFree(wins);
@@ -261,7 +341,7 @@ unmanaged_window (Window w)
 
   for (i = 0; unmanaged_window_list[i]; i++) 
     {
-      wname = get_window_name (w);
+      wname = get_wmname (w);
       if (!wname)
 	return 0;
       if (!strcmp (unmanaged_window_list[i], wname))
@@ -289,6 +369,79 @@ set_state (rp_window *win, int state)
 		   PropModeReplace, (unsigned char *)data, 2);
 }
 
+/* Get the WM state of the window. */
+long
+get_state (rp_window *win)
+{
+  long state = WithdrawnState;
+  Atom type;
+  int format;
+  unsigned long nitems;
+  unsigned long bytes_left;
+  long *data;
+
+  if (win == NULL) 
+    return state;
+
+  if (XGetWindowProperty (dpy, win->w, wm_state, 0L, 2L, 
+			  False, wm_state, &type, &format, 
+			  &nitems, &bytes_left, 
+			  (unsigned char **)&data) == Success && nitems > 0)
+    {
+      state = *data;
+      XFree (data);
+    }
+
+  return state;
+}
+
+static void
+move_window (rp_window *win)
+{
+  if (win->frame == NULL) 
+    return;
+
+  /* X coord. */
+  switch (win->gravity)
+    {
+    case NorthWestGravity:
+    case WestGravity:
+    case SouthWestGravity:
+      win->x = win->frame->x;
+      break;
+    case NorthGravity:
+    case CenterGravity:
+    case SouthGravity:
+      win->x = win->frame->x + (win->frame->width - win->border * 2) / 2 - win->width / 2;
+      break;
+    case NorthEastGravity:
+    case EastGravity:
+    case SouthEastGravity:
+      win->x = win->frame->x + win->frame->width - win->width - win->border;
+      break;
+    }
+
+  /* Y coord. */
+  switch (win->gravity)
+    {
+    case NorthEastGravity:
+    case NorthGravity:
+    case NorthWestGravity:
+      win->y = win->frame->y;
+      break;
+    case EastGravity:
+    case CenterGravity:
+    case WestGravity:
+      win->y = win->frame->y + (win->frame->height - win->border * 2) / 2 - win->height / 2;
+      break;
+    case SouthEastGravity:
+    case SouthGravity:
+    case SouthWestGravity:
+      win->y = win->frame->y + win->frame->height - win->height - win->border;
+      break;
+    }
+}
+
 /* Set a transient window's x,y,width,height fields to maximize the
    window. */
 static void
@@ -297,21 +450,28 @@ maximize_transient (rp_window *win)
   rp_window_frame *frame;
   int maxx, maxy;
 
+  /* We can't maximize a window if it has no frame. */
+  if (win->frame == NULL)
+    return;
+
   /* Set the window's border */
-  win->border = WINDOW_BORDER_WIDTH;
+  win->border = defaults.window_border_width;
 
   frame = win->frame;
 
-  /* Honour the window's maximum size */
-  if (win->hints->flags & PMaxSize)
+  /* Always use the window's current width and height for
+     transients. */
+  maxx = win->width;
+  maxy = win->height;
+
+  /* Fit the window inside its frame (if it has one) */
+  if (frame)
     {
-      maxx = win->hints->max_width;
-      maxy = win->hints->max_height;
-    }
-  else
-    {
-      maxx = win->width;
-      maxy = win->height;
+      PRINT_DEBUG ("frame width=%d height=%d\n", 
+		   frame->width, frame->height);
+
+      if (maxx + win->border * 2 > frame->width) maxx = frame->width - win->border * 2;
+      if (maxy + win->border * 2 > frame->height) maxy = frame->height - win->border * 2;
     }
 
   /* Make sure we maximize to the nearest Resize Increment specified
@@ -336,22 +496,6 @@ maximize_transient (rp_window *win)
 
   PRINT_DEBUG ("maxsize: %d %d\n", maxx, maxy);
 
-  /* Fit the window inside its frame (if it has one) */
-  if (frame)
-    {
-      win->x = frame->x - win->width / 2 
-	+ (frame->width - win->border * 2) / 2;
-      win->y = frame->y - win->height / 2
-	+ (frame->height - win->border * 2) / 2;
-    }
-  else
-    {
-      win->x = PADDING_LEFT - win->width / 2 
-	+ (win->scr->root_attr.width - PADDING_LEFT - PADDING_RIGHT - win->border * 2) / 2;
-      win->y = PADDING_TOP - win->height / 2 
-	+ (win->scr->root_attr.height - PADDING_TOP - PADDING_BOTTOM - win->border * 2) / 2;
-    }
-
   win->width = maxx;
   win->height = maxy;
 }
@@ -363,11 +507,13 @@ maximize_normal (rp_window *win)
 {
   rp_window_frame *frame;
   int maxx, maxy;
-  int off_x = 0;
-  int off_y = 0;
+
+  /* We can't maximize a window if it has no frame. */
+  if (win->frame == NULL)
+    return;
 
   /* Set the window's border */
-  win->border = WINDOW_BORDER_WIDTH;
+  win->border = defaults.window_border_width;
 
   frame = win->frame;
 
@@ -379,10 +525,8 @@ maximize_normal (rp_window *win)
     }
   else
     {
-      maxx = win->scr->root_attr.width 
-	- PADDING_LEFT - PADDING_RIGHT - win->border * 2;
-      maxy = win->scr->root_attr.height 
-	- PADDING_TOP - PADDING_BOTTOM - win->border * 2;
+      maxx = frame->width - win->border * 2;
+      maxy = frame->height - win->border * 2;
     }
 
   /* Fit the window inside its frame (if it has one) */
@@ -416,26 +560,6 @@ maximize_normal (rp_window *win)
     }
 
   PRINT_DEBUG ("maxsize: %d %d\n", maxx, maxy);
- 
-#ifdef MAXSIZE_WINDOWS_ARE_TRANSIENTS
-  if (win->hints->flags & PMaxSize && frame)
-    {
-      off_x = (frame->width - win->border * 2) / 2 - maxx / 2;
-      off_y = (frame->height - win->border * 2) / 2 - maxy / 2;
-    }
-#endif
-
-  /* Fit the window inside its frame (if it has one) */
-  if (frame)
-    {
-      win->x = frame->x + off_x;
-      win->y = frame->y + off_y;
-    }
-  else
-    {
-      win->x = PADDING_LEFT + off_x;
-      win->y = PADDING_TOP + off_y;
-    }
 
   win->width = maxx;
   win->height = maxy;
@@ -451,17 +575,21 @@ maximize (rp_window *win)
 
   /* Handle maximizing transient windows differently. */
   if (win->transient)
-    {
-      maximize_transient (win);
-    }
+    maximize_transient (win);
   else
-    {
-      maximize_normal (win);
-    }
+    maximize_normal (win);
 
-  /* Actually do the maximizing */
+  /* Reposition the window. */
+  move_window (win);
+
+  PRINT_DEBUG ("Resizing window '%s' to x:%d y:%d w:%d h:%d\n", window_name (win), 
+	       win->x, win->y, win->width, win->height);
+
+
+  /* Actually do the maximizing. */
   XMoveResizeWindow (dpy, win->w, win->x, win->y, win->width, win->height);
   XSetWindowBorderWidth (dpy, win->w, win->border);
+
   XSync (dpy, False);
 }
 
@@ -475,10 +603,28 @@ force_maximize (rp_window *win)
 
   maximize_normal(win);
 
-  /* Actually do the maximizing */
-  XMoveResizeWindow (dpy, win->w, win->x, win->y, win->width+1, win->height+1);
+  /* Reposition the window. */
+  move_window (win);
+
+  /* This little dance is to force a maximize event. If the window is
+     already "maximized" X11 will optimize away the event since to
+     geometry changes were made. This initial resize solves the
+     problem. */
+  if (win->hints->flags & PResizeInc)
+    {
+      XMoveResizeWindow (dpy, win->w, win->x, win->y,
+			 win->width + win->hints->width_inc, 
+			 win->height + win->hints->height_inc);
+    }
+  else
+    {
+      XResizeWindow (dpy, win->w, win->width + 1, win->height + 1);
+    }
+
+  /* Resize the window to its proper maximum size. */
   XMoveResizeWindow (dpy, win->w, win->x, win->y, win->width, win->height);
   XSetWindowBorderWidth (dpy, win->w, win->border);
+
   XSync (dpy, False);
 }
 
@@ -486,7 +632,7 @@ force_maximize (rp_window *win)
 void
 map_window (rp_window *win)
 {
-  PRINT_DEBUG ("Mapping the unmapped window %s\n", win->name);
+  PRINT_DEBUG ("Mapping the unmapped window %s\n", window_name (win));
 
   /* Fill in the necessary data about the window */
   update_window_information (win);
@@ -512,10 +658,10 @@ map_window (rp_window *win)
     {
       if (win->transient)
 	marked_message_printf (0, 0, MESSAGE_MAP_TRANSIENT, 
-			       win->number, win->name);
+			       win->number, window_name (win));
       else
 	marked_message_printf (0, 0, MESSAGE_MAP_WINDOW,
-			       win->number, win->name);
+			       win->number, window_name (win));
     }
 }
 
@@ -570,7 +716,7 @@ withdraw_window (rp_window *win)
 {
   if (win == NULL) return;
 
-  PRINT_DEBUG ("withdraw_window on '%s'\n", win->name);
+  PRINT_DEBUG ("withdraw_window on '%s'\n", window_name (win));
 
   /* Give back the window number. the window will get another one,
      if it is remapped. */
